from asyncore import read
from email.mime import audio
import json
import logging
import os
import socket
import threading
import time

from pathlib import Path

from barix.web.utils import readFromJSONFile, writeToJSONFile
from barix.web.stored_files import getDirectoryContentTree, sortDirectoryTreeList
from barix.web.audio_files import removeAudioFiles, getAudioFileDuration
from barix.web.exceptions import FileIsPlayingError, InvalidAudioFileError

import baco
#Baco logger
#baco.initializeLogger("/var/log/baco.log", baco.LogPriority.DEBUG, 5000000, 2)
#baco.initializeAudioLib()
import alsaaudio
from barix.web.uci import getValueOfUci
from utils import set_flag, clear_flag

# from scheduler import getSchedulerConfig, setSchedulerConfig
# from ipformer import getSourcesStatus

log = logging.getLogger('flask-backend')

# AUDIO_FILES_JSON_FILE = "/mnt/data/config/audioFiles.json"

# USB_FILES_JSON_FILE = "/tmp/usbFiles.json"
# PLAYLISTS_FILE_PATH = "/mnt/data/config/playlists.json"
# AUDIO_FILES_DIR = "/mnt/data/AudioFiles"
AUDIO_FILES_DIR = "/mnt/data/icpaging-master/raw_ringtones/"
AUDIO_FILES_JSON_FILE = AUDIO_FILES_DIR + "raw_ringtones.json"
FREE_STATION_NUMBERS = AUDIO_FILES_DIR + "free_station_numbers.json"
USB_MOUNTED_DIR = "/run/media/sda1"
#AUDIO_FILES_FORMAT = ["mp3", "aac", "flac", "ac3", "dts", "pcm", "vorbis"] # wav -> pcm; ogg -> vorbis; m4a -> aac
AUDIO_FILES_FORMAT = ["mp3", "wav", "zip", "ogg"]

def getLocalAudioFiles():
    try:
        localAudioFiles = readFromJSONFile(AUDIO_FILES_JSON_FILE)
    except FileNotFoundError:
        localAudioFiles=[]
    except Exception as e:
        log.error(e)
        raise e
    return localAudioFiles


"""
cleanDirectoryTree removes empty directory entries from the treeList provided
treeList is a list of dictionaries, where each dict represents an entry from the directory
It can be either a file or a directory. 
If it is a directory, it will have a key 'children' which value is a treeList.
"""
def cleanDirectoryTree(treeList):
    try:
        elemsToDelete = []
        for elem in treeList:
            if 'children' in elem.keys():
                if len(elem['children']) == 0:
                    # treeList.remove(elem)
                    elemsToDelete.append(elem)
                else:
                    cleanDirectoryTree(elem['children'])
                    if len(elem['children']) == 0:
                        # treeList.remove(elem)
                        elemsToDelete.append(elem)
        for elem in elemsToDelete:
            treeList.remove(elem)
    except Exception as e:
        log.error(e)
        raise e

def appendParamsToListElements(elemsList, paramsList, treePath=""):
    try:
        for elem in elemsList:
            if "children" in elem.keys():
                currentPath = os.path.join(treePath, elem["name"])
                appendParamsToListElements(elem["children"], paramsList, currentPath)
            else: # file
                for param in paramsList:
                    if param == "duration":
                        try:
                            elem["duration"] = getAudioFileDuration(os.path.join(USB_MOUNTED_DIR, treePath, elem["name"]))
                        except InvalidAudioFileError:
                            log.error('InvalidAudioFileError')
                            #TODO: remove from list
    except Exception as e:
        log.error("Following error occurred: %s",e)
        raise e

# def getPlaylists():
#     playlists = {}
#     try:
#         if (os.path.exists(PLAYLISTS_FILE_PATH)):
#             playlists = readFromJSONFile(PLAYLISTS_FILE_PATH)
#     except Exception as e:
#         log.error(e)
#         raise e
#     else:
#         return playlists

def deleteAudioFiles(filesIdList):
    try:
        # # first, check if file is playing (can't delete a file that is playing)
        # sourcesStatus = getSourcesStatus()
        # if "status" in sourcesStatus:
        #     for status in sourcesStatus["status"]:
        #         if status["priority"] == 0 and status["type"] == "local_file":
        #             if status["state"] == "playing":
        #                 raise FileIsPlayingError
        # # then, remove file from playlists and scheduler if it is being used there
        # playlistsData = getPlaylists()
        # schedulerData = getSchedulerConfig()
        # for fileId in filesIdList:
        #     for playlistId in playlistsData:
        #         if 'filesList' in playlistsData[playlistId].keys():
        #             playlistsData[playlistId]["filesList"] = [x for x in playlistsData[playlistId]["filesList"] if x != fileId]
        #     if 'fixed_time' in schedulerData.keys():
        #         schedulerData["fixed_time"] = [x for x in schedulerData["fixed_time"] if x["fileId"] != fileId]
        #     if 'rotational' in schedulerData.keys() and 'filesList' in schedulerData["rotational"].keys():
        #         schedulerData["rotational"]["filesList"] = [x for x in schedulerData["rotational"]["filesList"] if x["fileId"] != fileId]
        # with open(PLAYLISTS_FILE_PATH, 'w') as outfile:
        #     json.dump(playlistsData, outfile)
        # setSchedulerConfig(schedulerData)

        # finally, remove from the audiofiles json file and from storage
        filesList = [os.path.join(AUDIO_FILES_DIR, x) for x in filesIdList]

        for fileID in filesIdList:
                st_number_to_release = getStationNumberFromFileID(fileID)
                if st_number_to_release != 'NA':
                    releaseStationNumber(int(st_number_to_release))
        res = removeAudioFiles(AUDIO_FILES_JSON_FILE, filesList)
        refreshAvailableStationNumbers()            
        return res
    except FileIsPlayingError:
        raise FileIsPlayingError
    except Exception as e:
        log.error(e)
        raise e


local_player = None

def playAudioFile(filename: str = '', sample_rate : int = 44100):

    log.info("Trying to play \"{}\"".format(filename))
    
    global local_player
    # output_audio_device = getValueOfUci("icpaging-master", "audio", "output_device")  # plughw:2,0
    output_audio_device = getValueOfUci("application", "audio", "silent_playback")  # plug:master1

    urlAudio = get_filepath_from_filename(filename)

    if urlAudio != 'NA':
        try:
            local_player = baco.SimplePlayer(urlAudio, output_audio_device, sample_rate)
            local_player.setCallback(playerStatusCallback)
            local_player.connect()
            log.info("Baco player started. Using audio device: %s", output_audio_device)
            log.info("New baco player: %s", local_player)
            set_flag('playing')
            file_info = {   
                "status": [{
                            "id": 0, 
                            "filename": filename, 
                            "filepath": urlAudio,
                            "priority": 0, 
                            "type": "local_file", 
                            "state": "playing",
                            "output_audio_device": output_audio_device 
                        }],
                "timestamp": time.time()}
            setPlayingStatus(file_info)

            return "OK"
        except FileIsPlayingError:
            raise FileIsPlayingError
    else:
        stopPlayingAudioFile()
    return "NOK"


# TODO: Needs testing. Should be called from bacolib
def playerStatusCallback(alive):    
    if alive:
        log.info(f"File playing")
    else:
        setFinishPlaying()
        log.info(f"Finished playing")


def stopPlayingAudioFile():
    global local_player

    if local_player is not None:
        setFinishPlaying()
        local_player.disconnect()
        log.info("Disconnected from baco SimplePlayer: %s",local_player)
        local_player = None        
    else:
        log.info("baco SimplePlayer already None")

    if local_player is None:
        return True
    else:
        log.error(f"Local player is still active: {local_player}")
        return False

def setPlayingStatus(new_info):
    writeToJSONFile("/var/tmp/paging_master/playing_status.json", new_info)

#TODO: Needs implementaiton and testing
def updatePlayingStatus(update_info):
    current_file_info = readFromJSONFile("/var/tmp/paging_master/playing_status.json")
    log.info("Before updating file")

    log.info("Update info: %s", update_info)

    try:
        for item in current_file_info:
            log.info("Item: %s", item)
            if item == "status":
                for section in current_file_info["status"][0]:
                    if section in update_info['status'][0]:
                        current_file_info["status"][0][section] = update_info["status"][0][section]
                        log.info("Updated value for -> %s : %s", section, current_file_info["status"][0][section])

            if item == "timestamp":
                if "timestamp" in update_info:
                    current_file_info["timestamp"] = update_info["timestamp"]

        setPlayingStatus(current_file_info)
    except Exception as e:
        log.warning("Update playing status failed: %s", e)

def setFinishPlaying():
    global local_player
    update_info = {   
            "status": [{ 
                        "state": "finished",
                    }],
            "timestamp": time.time()}
    updatePlayingStatus(update_info)
    local_player.disconnectAudio()
    # local_player = None
    log.info("Disconnected audio from player")

#TODO: Needs implementaiton and testing
def getPlayingStatus():
    playing_status = ""
    try:
        if os.path.isfile("/var/tmp/paging_master/playing_status.json"): 
            playing_status = readFromJSONFile("/var/tmp/paging_master/playing_status.json")  
            # log.info("Returning following: %s", playing_status)         
        return playing_status
    except Exception as e:
        log.error(e)
        raise e


def get_filepath_from_filename(filename: str = ''):
    local_files = getLocalAudioFiles()

    for elem in local_files:
        if elem["filename"] == filename:
            return AUDIO_FILES_DIR + elem["file_id"]
    return "NA"

def getStationNumberFromFileID(file_id: str = ''):
    local_files = getLocalAudioFiles()

    for elem in local_files:
        if elem["file_id"] == file_id:
            st_number = elem["station_number"]
            log.info("Need to release ID: %s", st_number)
            return elem["station_number"]
    return "NA"


# TODO: Needs testing
def refreshAvailableStationNumbers():
    local_files = getLocalAudioFiles()
    occupied_station_numbers = [elem["station_number"] for elem in local_files]
    
    log.info("Occupied stations: %s", sorted([int(x) for x in occupied_station_numbers]))
    if len(occupied_station_numbers) == 0 or not os.path.exists(FREE_STATION_NUMBERS):
        generateAllStationNumbers()
        
    free_station_numbers = readFromJSONFile(FREE_STATION_NUMBERS)
    new_available_station_numbers = [station for station in free_station_numbers if station not in occupied_station_numbers]

    writeToJSONFile(filePath=FREE_STATION_NUMBERS, payloadJSON=new_available_station_numbers)


# TODO: Needs testing
def generateAllStationNumbers(start: int = 1, end: int = 129):
    """
        This function needs to be called at least once at the start of the program.
    """
    if os.path.exists(FREE_STATION_NUMBERS):
        os.remove(FREE_STATION_NUMBERS)
    station_numbers = [str(station) for station in range(start, end)]
    writeToJSONFile(filePath=FREE_STATION_NUMBERS, payloadJSON=station_numbers)


# TODO: Needs testing
def getFreeStationNumber(requested_station_number: str = ''):
    """
        To avoid data inconsistency, the returned number is only removed from the free list,
        once it has been actually stored inside the raw_ringones.json file.
        Until then it is still considered to be free. 
    """

    refreshAvailableStationNumbers()
    free_station_numbers = readFromJSONFile(FREE_STATION_NUMBERS)
    int_stations = [int(x) for x in free_station_numbers]

    if requested_station_number == '':
        station_number = str(min(int_stations))
        log.info(f"Returning minimum available station number: {station_number}")
        return station_number
    
    if requested_station_number in free_station_numbers:
        log.info(f"Requested station number {requested_station_number} granted")
        return requested_station_number

    log.warning(f"Station number {requested_station_number} is already taken")
    raise Exception


# TODO: Needs testing
def releaseStationNumber(station_number: int = -1):
    if (station_number == -1) or (station_number not in range(1, 129)):
        log.warning("Not a valid station number")        
        return False

    log.info(f"Station number {station_number} can be released")
    free_station_numbers = readFromJSONFile(FREE_STATION_NUMBERS)
    free_station_numbers.append(str(station_number))
    writeToJSONFile(filePath=FREE_STATION_NUMBERS, payloadJSON=free_station_numbers)
    log.info("Done!")
    return True


#TODO: Needs testing
def changeStationNumber(file_info: str = '', new_number: str = ''):
    log.info("File info: %s", file_info)
    if releaseStationNumber(int(file_info['station_number'])):
        to_change = getFreeStationNumber(new_number)
        file_info['station_number'] = to_change
        return True
    return False
