import json
import logging
import os
import time
import subprocess
import shlex
from flask import Blueprint, Response, request

from uci import UciException

from barix.web.uci import setUciConfigs
from barix.web.auth import login_required

from restart_services import createListOfServicesToRestart

log = logging.getLogger('flask-backend')

bp = Blueprint('icpgm_API', __name__)

GROUPS_FILE = "/barix/apps/icmaster/src/groups.json"

@bp.route('/api/rest/v1/icpaging/master/settings', methods=['PUT'])
@login_required
def handleSetICPagingMasterSettings():
    try:
        data = json.loads(request.data)
        log.debug("Device settings to set: {}".format(data))
    except Exception as e:
        log.error(e)
        return Response("Invalid request body", status=400)
    try:
        jsonUcis = data['ucis']
        setUciConfigs(jsonUcis, commit=data['commit'], restartServices=False,
                                       selectServicesToRestartCallback=createListOfServicesToRestart)
    except UciException as e:
        return Response("Invalid " + e, status=400)
    except Exception as e:
        log.error(e)
        return Response(status=500)

    try:
        groupsJson = data['groups']
        groups = {}
        for elem in groupsJson:
            groups[elem["id"]] = elem["clients"]
        f = open(GROUPS_FILE, 'w')
        f.write(json.dumps(groups))
        f.close()
        # restart service
        cmd = "/etc/init.d/icpaging-master restart"
        log.info("Executing: {}".format(cmd))
        proc = subprocess.Popen(shlex.split(cmd), shell=False)
        proc.wait()
        res = proc.returncode
        if res == 0:
            return Response(status=200)
        else:
            log.error("Restarting main app return code: {}".format(res))
            return Response(status=500)
    except Exception as e:
        log.error(e)
        return Response(status=500)



@bp.route('/api/rest/v1/icpaging/master/groups', methods=['GET'])
@login_required
def handleGetPagingGroups():
    groups = []
    try:
        if os.path.isfile(GROUPS_FILE):
            with open(GROUPS_FILE) as json_file:
                groupsContent = json.load(json_file)
                for key in groupsContent:
                    group = {}
                    group["id"] = key
                    group["clients"] = groupsContent[key]
                    groups.append(group)
        else:
            groups = [
                {"id": "1", "clients": []},
                {"id": "2", "clients": []},
                {"id": "3", "clients": []},
                {"id": "4", "clients": []},
                {"id": "5", "clients": []},
                {"id": "6", "clients": []},
                {"id": "7", "clients": []},
                {"id": "8", "clients": []},
            ]
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(json.dumps(groups), status=200)