import logging
from logging.handlers import RotatingFileHandler
import pathlib


"""
Init logger
"""


def configLogger(name, log_file_path, log_max_size, log_backup_count):
    log = logging.getLogger(name)
    log.setLevel(logging.DEBUG)
    rot_handler = RotatingFileHandler(log_file_path, maxBytes=log_max_size, backupCount=log_backup_count)
    format1 = logging.Formatter('%(asctime)s  %(levelname)s  %(filename)s:%(lineno)s  %(message)s')
    rot_handler.setFormatter(format1)
    log.addHandler(rot_handler)


# TODO: Not tested yet
def get_logger(name: str = "",
               config: str = "",
               log_file: str = "/var/log/new_logger_test",
               log_max_size: int = 2*1024*1024,
               log_backup_count: int = 2):

    if config == "":
        print(f"Creating default logger for {name}")
        logger = logging.getLogger(name)
        logger.setLevel(logging.INFO)
        # Initialize directory structure
        log_file = pathlib.Path(log_file)
        log_file.touch(exist_ok=True)
        rot_handler = RotatingFileHandler(log_file, maxBytes=log_max_size, backupCount=log_backup_count)

        # create formatter, to get path: - [%(pathname)s]
        formatter = logging.Formatter(
            "%(asctime)s barix-ipam400 " + name + "[%(lineno)d] - %(levelname)s - %(message)s [%(threadName)s]",
            datefmt="%b %d %H:%M:%S",
        )

        # add formatter to rot_handler
        rot_handler.setFormatter(formatter)

        # add rot_handler to logger
        logger.addHandler(rot_handler)

    else:
        print("About to load .conf file: %s", config)
        logging.config.fileConfig(config)
        logger = logging.getLogger(name)
        print("Loaded the .conf file")
    return logger
