import json
import os
from pathlib import Path
from logger import get_logger


PAGING_MASTER_TMP_PATH = "/var/tmp/paging_master/"
STATE_FILE = "paging_state.json"
INPUT_LEVEL = "input_level.json"
OUTPUT_LEVEL = "output_level.json"

logger = get_logger(name="PagingState")


def get_paging_state() -> str:

    device_state_file = os.path.join(PAGING_MASTER_TMP_PATH, STATE_FILE)

    logger.info("Trying to get the device state")

    try:
        fact_info_json = read_json(device_state_file)
        if "STATE" in fact_info_json.keys():
            paging_state = fact_info_json["STATE"]
        else:
            paging_state = "Unknown"
        logger.info(f"Returning state: {paging_state}")
        return paging_state
    except Exception as e:
        logger.error("Exception found: {}".format(e))
    return ""


def get_sound_level(sound_level_direction: str) -> int:

    sound_level_file = os.path.join(PAGING_MASTER_TMP_PATH, INPUT_LEVEL)

    if sound_level_direction == "output_level":
        sound_level_file = os.path.join(PAGING_MASTER_TMP_PATH, OUTPUT_LEVEL)
    
    try:
        fact_info_json = read_json(sound_level_file)
        if "LEVEL" in fact_info_json.keys():
            sound_level = fact_info_json["LEVEL"]
        else:
            sound_level = 0
        logger.info(f"Returning sound level: {sound_level}")
        return sound_level
    except Exception as e:
        logger.error("Exception found: {}".format(e))
    return 0


def read_json(file_name):
    info_json = "{}"
    if Path(file_name).exists():
        with open(file_name, "r") as f:
            info_json = json.load(f)

    return info_json
