import logging
import json
import os

from flask import Blueprint, request, Response

from barix.web.auth import login_required
from barix.web.system_info import *

from paging_master_info import get_paging_state, get_sound_level

bp = Blueprint('paging_master_info_API', __name__)

log = logging.getLogger('flask-backend')


@bp.route('/api/rest/v1/icpaging/master/info', methods=['GET'])
@login_required
def getPagingMasterInfo():
    result_dict = {}
    try:
        settingsRequested = request.args
        # log.info("Requesting paging master info: {}".format(settingsRequested))

        for requestedSetting in settingsRequested:
            if requestedSetting == "paging_state":
                result_dict["paging_state"] = get_paging_state()

            elif requestedSetting == "input_level":
                result_dict["input_level"] = get_sound_level("input_level")

            elif requestedSetting == "output_level":
                result_dict["output_level"] = get_sound_level("output_level")

            else:
                log.error("Unrecognized setting {}".format(requestedSetting))
                return Response("Unrecognized setting {}".format(requestedSetting), status=400)
                
    except Exception as e:
        log.error(e)
        return Response(status=500)

    # log.info("Returning system info: {}".format(result_dict))
    return Response(json.dumps(result_dict), status=200)