import logging

from barix.web.certificates import generateSSLCerts
from barix.web.auth import generateNewSessionKey
from barix.web.uci import getValueOfUci
from utils import set_flag

log = logging.getLogger('flask-backend')


def createListOfServicesToRestart(ucis_list):

    # perform additional actions before restarting services
    checkIfWebProtocolChanged(ucis_list)

    # create list of services
    servicesToRestart = []
    for uci in ucis_list:

        try:
            config = uci[0]
            section = uci[1]
            option = uci[2]
            
            # log.info(f">>>>>>    Config: {config}, section: {section}, option: {option}    <<<<<<<<<")
            if config == "aes67":
                pass # services are restarted in another place
            elif config == "application":
                # The app needs to be restarted as the settings are taken in the init script
                if section == "audio":
                    servicesToRestart.append("icmaster_main_app")
                if section == "audio" and option == "volume":
                    servicesToRestart.append("{}.{}.{}".format(config, section, option))
                # if section == "main_config" and option == "mode":
                #     servicesToRestart.append('reboot')
            elif config == "avahi":
                servicesToRestart.append("S21avahi-daemon")
            elif config == "device":
                # log.debug("Changes on device uci config do not require do restart any service")
                pass
            elif config == "dropbear":
                if section == "RunCtl" and option == "enable":
                    servicesToRestart.append("dropbear")
            elif config == "flexa_agent":
                pass
            elif config == "flexa_app":
                #    listOfServicesToRestart.append("S92flexa_app")
                pass
            # elif config == "flexa_ip_former":
            #     if section == "tpa400" and option == "level":
            #         servicesToRestart.append("reboot")
            #     else:
            #         #servicesToRestart.append("S98run-flexa")
            #         servicesToRestart.append("ipac_main_app")
            elif config == "icpaging-master":
                # The app needs to be restarted as the settings are taken in the init script
                if section == "audio" and (option == 'sample_rate' or option == 'audio_format'):
                    # Inform the main app which files need to be re-encoded
                    set_flag("ringtones")
                servicesToRestart.append("icmaster_main_app")
            elif config == "httpd":
                section = uci[1]
                option = uci[2]
                if section == "webserver" and option == "protocol":
                    # listOfServices.append("S50httpd") # restart lighttpd for now too because of update
                    # servicesToRestart.append("backend_flask") # restart backend from new_webui
                    servicesToRestart.append("icmaster_webui")
                    # listOfServices.append('reboot') # temp
            elif config == "informacast":
                pass
            elif config == "licensing":
                pass
            elif config == "network":
                servicesToRestart.append('reboot')
            elif config == "ntp":
                servicesToRestart.append("S20ntpd")
            elif config == "poe":
                servicesToRestart.append('reboot')
            elif config == "proxy_config":
                pass
            elif config == "rtc":
                pass
            elif config == "security":
                # log.debug("Changes on security uci config do not require do restart any service")
                pass
            elif config == "sonic_ip":
                # log.debug("Changes on sonic_ip uci config do not require do restart any service")
                # listOfServices.append("S13sonic-ip")
                pass
            elif config == "speaker":
                servicesToRestart.append('reboot')
            elif config == "syslogd":
                pass
            elif config == "timezone":
                servicesToRestart.append('S20timezone-handler')
            else:
                log.error("Unrecognized UCI config: {}".format(config))
        except Exception as e:
            log.error(e)
            raise e

    return servicesToRestart


def checkIfWebProtocolChanged(ucisChanged):
    try:
        # check if web protocol changed to HTTPS
        for uciTuple in ucisChanged:
            if ('httpd', 'webserver', 'protocol') == uciTuple:
                generateNewSessionKey()
                if getValueOfUci('httpd', 'webserver', 'protocol') == 'https':
                    generateSSLCerts()
            break
    except Exception as e:
        raise e
