import json
import logging
from flask import Blueprint, Response, request
from uci import UciException


from barix.web.webui_password import setWebUiPassword
from barix.web.exceptions import InvalidRequestBodyContentError, InvalidOldPasswordError
from barix.web.uci import getValueOfUci, formatUciTupleIntoString, setUciConfigs
from barix.web.auth import login_required, generateNewSessionKey
from restart_services import createListOfServicesToRestart


log = logging.getLogger('flask-backend')

bp = Blueprint('settings_API', __name__)

@bp.route('/api/rest/v1/settings', methods=['PUT'])
@login_required
def setDeviceSettings():
    try:
        data = json.loads(request.data)
        log.debug("Device settings to set: {}".format(data))
    except Exception as e:
        log.error(e)
        return Response("Invalid request body", status=400)

    if 'pwdSettings' in data.keys():
        try:
            setWebUiPassword(data['pwdSettings'])
        except InvalidRequestBodyContentError:
            return Response("Invalid request body", status=400)
        except InvalidOldPasswordError:
            return Response("Old password is invalid", status=400)
        except Exception as e:
            log.error(e)
            return Response(status=500)

    if 'ucis' in data.keys():
        if not 'commit' in data.keys():
            log.error("commit not in uciSettings")
            return Response("Invalid request body", status=400)
        jsonUcis = data['ucis']
        try:
            ucisConfigured = setUciConfigs(jsonUcis, commit=data['commit'], restartServices=True, selectServicesToRestartCallback=createListOfServicesToRestart)
            uciListSet = []
            for uciTuple in ucisConfigured:
                uciListSet.append(formatUciTupleIntoString(uciTuple))
            return_msg = {"ucisSet": uciListSet}
            return Response(json.dumps(return_msg), status=200)
        except UciException as e:
            return Response("Invalid "+e, status=400)
        except Exception as e:
            log.error(e)
            return Response(status=500)

    return Response(status=200)

@bp.route('/api/rest/v1/settings/volume', methods=['PUT'])
@login_required
def setVolume():
    try:
        uciList = json.loads(request.data)
    except Exception as e:
        log.error(e)
        return Response("Bad formatted request body", status=400)
    try:
        setUciConfigs(uciList, commit=True, restartServices=True, selectServicesToRestartCallback=createListOfServicesToRestart)
    except Exception as e:
        log.error(e)
        return Response(status=500)
    except UciException as e:
        return Response("Invalid " + e, status=400)
    else:
        return Response(status=200)