import logging
import json
import os

from flask import Blueprint, request, Response

from barix.web.auth import login_required
from barix.web.system_info import *

bp = Blueprint('system_info_API', __name__)

log = logging.getLogger('flask-backend')

@bp.route('/api/rest/v1/system/info', methods=['GET'])
@login_required
def getSystemInfo():
    result_dict = {}
    try:
        settingsRequested = request.args
        log.info("Requesting system info: {}".format(settingsRequested))

        for requestedSetting in settingsRequested:
            if requestedSetting == "device_type":
                # get HW_type
                dev_type = getDeviceType()
                result_dict["device_type"] = dev_type

            elif requestedSetting == "device_type_ID":
                # get HW_type_id
                dev_type_id = getDeviceTypeID()
                result_dict["device_type_ID"] = dev_type_id

            elif requestedSetting == "IPAM_type":
                # get IPAM_type
                IPAM_type = getIPAMType()
                result_dict["IPAM_type"] = IPAM_type

            elif requestedSetting == "IPAM_type_ID":
                # get IPAM_type_id
                IPAM_type_id = getIPAMTypeID()
                result_dict["IPAM_type_ID"] = IPAM_type_id

            elif requestedSetting == "mac_addr":
                # get mac address
                mac_addr = getMACaddr()
                result_dict["mac_addr"] = mac_addr

            elif requestedSetting == "ip_addr":
                # get IP_addr
                ip_addr = getIPaddr('eth0')
                result_dict["ip_addr"] = ip_addr

            elif requestedSetting == "netmask":
                # get netmask
                netmask = getNetmask()
                result_dict["netmask"] = netmask

            elif requestedSetting == "default_gateway":
                # get defaultGW
                defaultGW = getDefaultGW()
                result_dict["default_gateway"] = defaultGW

            elif requestedSetting == "DSN_servers":
                # get DNS_servers
                dns_servers = getDnsServers()
                result_dict["DSN_servers"] = dns_servers

            elif requestedSetting == "FW":
                # get FW
                #TODO: get from manifest file
                result_dict["FW"] = "IP Speaker"

            elif requestedSetting == "fw_version":
                # get fwVersion
                fw_version = getFWVer()
                result_dict["fw_version"] = fw_version

            elif requestedSetting == "rootFSDate":
                rootFSDate = getRootFSCompilationDate()
                result_dict["rootFSDate"] = rootFSDate

            elif requestedSetting == "webUIVer":
                webUiVer = getWebUiVersion()
                result_dict["webUIVer"] = webUiVer

            elif requestedSetting == "app":
                # get Application
                result_dict["app"] = getApplication()

            elif requestedSetting == "app_ver":
                # get Application version
                result_dict["app_ver"] = getAppVersion()

            elif requestedSetting == "kernelVer":
                # get kernelVer
                kernelVer = getKernelVer()
                result_dict["kernelVer"] = kernelVer

            elif requestedSetting == "bootloaderVer":
                # get bootloaderVer
                bootloaderVer = getBootloaderVer()
                result_dict["bootloaderVer"] = bootloaderVer

            elif requestedSetting == "sys_time":
                # get system time
                result_dict["sys_time"] = getSystemTime()

            elif requestedSetting == "uptime":
                # get uptime
                result_dict["uptime"] = getUpTime()

            elif requestedSetting == "license":
                # get lic data
                result_dict['licenses'] = getLicenses()

            elif requestedSetting == "massStorageDev":
                # get storage devices
                result_dict["massStorageDev"] = getMassStorageDevices()

            elif requestedSetting == "HW_type_ID":
                # get HW Type ID
                result_dict["HW_type_ID"] = getHWTypeID()

            elif requestedSetting == "conn_status":
                result_dict["conn_status"] = getWiredConnStatus()

            elif requestedSetting == "poe_power_source":
                # get PoE Power Source
                result_dict["poe_power_source"] = getPoEConfig()

            elif requestedSetting == "poe_speaker_impedance":
                # get PoE Speaker Impedance
                result_dict["poe_speaker_impedance"] = getPoESpkImp()

            elif requestedSetting == "poe_amp_gain":
                # get PoE Amplifier Gain
                result_dict["poe_amp_gain"] = getPoEAmpGain()

            elif requestedSetting == "otw_status":
                # get Amplifier over temperature status
                result_dict["otw_status"] = getPoEOverTempStatus()

            elif requestedSetting == "amp_temp":
                result_dict["amp_temp"] = getAmplifierTemperature()

            elif requestedSetting == "sensor_temp":
                result_dict["sensor_temp"] = getTemperatureFromSensor()

            elif requestedSetting == "open_source_licenses":
                result_dict["open_source_licenses"] = getOpenSourceLicenses()

            elif requestedSetting == "storage_size":
                result_dict["storage_size"] = getStorageSize()                

            else:
                log.error("Unrecognized setting {}".format(requestedSetting))
                return Response("Unrecognized setting {}".format(requestedSetting), status=400)
    except Exception as e:
        log.error(e)
        return Response(status=500)

    log.info("Returning system info: {}".format(result_dict))
    return Response(json.dumps(result_dict), status=200)

@bp.route('/api/rest/v1/system/type', methods=['GET'])
@login_required
def getSystemType():
    return_msg = {}

    try:
        return_msg['HW_type_ID'] = getHWTypeID()

        if os.path.exists('/etc/barix/ussi/BARIX_USSI.json'):
            return_msg['hasSNMPSettings'] = True

        if os.path.exists('/tmp/poe-detect'):
            return_msg['hasPoESettings'] = True

        if os.path.exists('/tmp/temp_sensor'):
            return_msg['hasTempSensor'] = True
    except Exception as e:
        log.error(e)
        return Response(status=500)

    log.info("Returning system type: {}".format(return_msg))
    return Response(json.dumps(return_msg), status=200)



