import os
from logger import get_logger
from barix.web.utils import readFromJSONFile, writeToJSONFile

logger = get_logger(name="Utils", log_file="/var/log/utils_backend_log")

VALID_FLAGS = {
    "ringtones": "/var/tmp/paging_master/encode.flag",
    "reset_defaults": "/etc/flags/reset_defaults.flag",
    "update_panel": "/etc/flags/update_panel.flag",
    "playing": "/var/tmp/paging_master/playing_status.json",
    }


def set_flag(flag_type: str = "ringtones") -> bool:
    """
        When audio settings are changed, set the /var/tmp/paging_master/encode.flag to re-encode the ringtones.
        Later on additional flags could be added for other audio files, or whatever needed. Add them to the
        VALID_FLAGS dict, where the key is the flag type, and the value is where the flag file needs to be created.
        Clearing the flags should be taken care of in the main device app.
    :param flag_type: Type of flag.
    :return: True if the requested flag was set, false otherwise
    """
    logger.info(f"Checking if the flag to set [{flag_type}] is valid.")
    for flag, flag_path in VALID_FLAGS.items():
        if flag == flag_type:
            try:
                os.popen(f'touch {flag_path}')
                logger.info("The following flag has been set: %s", flag_path)
                return True

            except Exception as e:
                logger.error("Following exception occurred: %s", e)
                return False
    logger.info("Flag to set not valid!")
    return False


def clear_flag(flag_type: str = "ringtones") -> bool:
    logger.info(f"Checking if the flag to clear [{flag_type}] is valid.")
    for flag, flag_path in VALID_FLAGS.items():
        if flag == flag_type:
            try:
                os.popen(f'rm {flag_path}')
                logger.info("The following flag has been cleared: %s", flag_path)
                return True
            except Exception as e:
                logger.error("Following exception occurred: %s", e)
                return False                
    logger.info("Flag to clear not valid!")
    return False
