from utils import read_json, get_uci
from nextion.nextion_enum import KeyID

BUTTON_ID_MAPPING = get_uci("icpaging-master", "main", "button_id_mapping")  # 'intuitive' or 'legacy'


def make_group_mapping():
    group_mapping = {}
    try:
        group_json = read_json("groups.json")
        for i in range(1, 8 + 1):
            group_mapping[f"GROUP_{i}_BUTTON"] = group_json[f"{i}"]
        return group_mapping
    except Exception as e:
        print(f"[mapping.py] Exception occurred: {e}")
        for i in range(1, 8 + 1):
            group_mapping[f"GROUP_{i}_BUTTON"] = []
        return group_mapping


def make_station_mapping():
    stations_map = {}
    default_ids = 17
    try:
        for key in KeyID:
            # print(f"[mapping.py] Current key: {key}, value: {key._value_}")
            if "STATION" in key.name:
                if BUTTON_ID_MAPPING == "intuitive":
                    station_number = int(str(key.name).split("_")[1])
                elif BUTTON_ID_MAPPING == "legacy":
                    # TODO: Inside the enum, return key and value with __str__.
                    #  Would require numerous chances in different nextion files
                    station_number = int(key._value_) + 1
                    # print(f"Current station: {key.name}, station number: {station_number}")
                else:
                    station_number = default_ids
                    print(f"[mapping.py]Unknown ID mapping. Using default values. "
                          f"Station: {key.name}, ID: {station_number}")
                    default_ids += 1
                stations_map[key.name] = station_number
    except Exception as e:
        print(f"[mapping.py] Exception occurred: {e}")
        stations_map = STATION_MAPPING_DEFAULT.copy()

    return stations_map


GROUP_MAPPING = make_group_mapping()

STATION_MAPPING = make_station_mapping()
STATION_MAPPING_DEFAULT = {
    "STATION_1_BUTTON": 1,
    "STATION_2_BUTTON": 2,
    "STATION_3_BUTTON": 3,
    "STATION_4_BUTTON": 4,
    "STATION_5_BUTTON": 5,
    "STATION_6_BUTTON": 6,
    "STATION_7_BUTTON": 7,
    "STATION_8_BUTTON": 8,
    "STATION_9_BUTTON": 9,
    "STATION_10_BUTTON": 10,
    "STATION_11_BUTTON": 11,
    "STATION_12_BUTTON": 12,
    "STATION_13_BUTTON": 13,
    "STATION_14_BUTTON": 14,
    "STATION_15_BUTTON": 15,
    "STATION_16_BUTTON": 16,
    "STATION_17_BUTTON": 17,
    "STATION_18_BUTTON": 18,
    "STATION_19_BUTTON": 19,
    "STATION_20_BUTTON": 20,
    "STATION_21_BUTTON": 21,
    "STATION_22_BUTTON": 22,
    "STATION_23_BUTTON": 23,
    "STATION_24_BUTTON": 24,
    "STATION_25_BUTTON": 25,
    "STATION_26_BUTTON": 26,
    "STATION_27_BUTTON": 27,
    "STATION_28_BUTTON": 28,
    "STATION_29_BUTTON": 29,
    "STATION_30_BUTTON": 30,
    "STATION_31_BUTTON": 31,
    "STATION_32_BUTTON": 32,
    "STATION_33_BUTTON": 33,
    "STATION_34_BUTTON": 34,
    "STATION_35_BUTTON": 35,
    "STATION_36_BUTTON": 36,
    "STATION_37_BUTTON": 37,
    "STATION_38_BUTTON": 38,
    "STATION_39_BUTTON": 39,
    "STATION_40_BUTTON": 40,
    "STATION_41_BUTTON": 41,
    "STATION_42_BUTTON": 42,
    "STATION_43_BUTTON": 43,
    "STATION_44_BUTTON": 44,
    "STATION_45_BUTTON": 45,
    "STATION_46_BUTTON": 46,
    "STATION_47_BUTTON": 47,
    "STATION_48_BUTTON": 48,
}
