from enum import IntEnum, unique
from utils import get_uci

PAGES_TEMP_OFFSET = 1000


class NextionPagingState(IntEnum):
    IDLE = 1
    INCOMING_CALL_REQUEST = 2
    WAIT = 3
    IN_ACTIVE_CALL = 4
    PAGING_ACTIVE = 5
    UNATTENDED_MODE = 6
    RECORDING_AWAY_MESSAGE = 7
    USE_AWAY_MESSAGE = 8
    CALL_FORWARDING_MODE = 9
    SENDING_CUSTOM_MESSAGE = 10
    LOCAL_PLAYBACK = 11
    MESSAGE = 12
    RECORDING_CUSTOM_MESSAGE = 13
    FINISH_OPERATION = 14

    def __str__(self):
        return f" Paging State is {self.name}"


class KeyStatus(IntEnum):
    RELEASED = 0
    PRESSED = 1

    def __repr__(self):
        return f" {self._name_}"

    def __str__(self):
        return f" {self._name_}"


@unique
class KeyID(IntEnum):
    # Since it is an Enum, it needs to have a unique value. This is why 1000 is taken to be 0, 1001 is 1 and so on.
    number_pages = int(get_uci("icpaging-master", "main", "panel_pages")) + PAGES_TEMP_OFFSET

    # Control buttons.
    ACCEPT_CALL_BUTTON = 0
    PAGE_BUTTON = 1
    CLOSE_CALL_BUTTON = 2
    # Each new page adds 48 to the buttons. 0 pages -> 64, 1 page -> 112, 2 pages -> 160 ...
    RELAY_BUTTON = ((number_pages - PAGES_TEMP_OFFSET) * 48) + 64

    AWAY_BUTTON = 4
    RECORD_BUTTON = 7
    CANCEL_SELECTION_BUTTON = 3

    # Checkbox buttons
    CUSTOM_MESSAGE_CHECKBOX = 6
    AWAY_MESSAGE_CHECKBOX = 5

    # Group buttons
    GROUP_1_BUTTON = 8
    GROUP_2_BUTTON = 9
    GROUP_3_BUTTON = 10
    GROUP_4_BUTTON = 11
    GROUP_5_BUTTON = 12
    GROUP_6_BUTTON = 13
    GROUP_7_BUTTON = 14
    GROUP_8_BUTTON = 15

    # Station IDs Page 0
    STATION_1_BUTTON = 16
    STATION_2_BUTTON = 17
    STATION_3_BUTTON = 18
    STATION_4_BUTTON = 19
    STATION_5_BUTTON = 20
    STATION_6_BUTTON = 21
    STATION_7_BUTTON = 22
    STATION_8_BUTTON = 23
    STATION_9_BUTTON = 24
    STATION_10_BUTTON = 25
    STATION_11_BUTTON = 26
    STATION_12_BUTTON = 27
    STATION_13_BUTTON = 28
    STATION_14_BUTTON = 29
    STATION_15_BUTTON = 30
    STATION_16_BUTTON = 31
    STATION_17_BUTTON = 32
    STATION_18_BUTTON = 33
    STATION_19_BUTTON = 34
    STATION_20_BUTTON = 35
    STATION_21_BUTTON = 36
    STATION_22_BUTTON = 37
    STATION_23_BUTTON = 38
    STATION_24_BUTTON = 39
    STATION_25_BUTTON = 40
    STATION_26_BUTTON = 41
    STATION_27_BUTTON = 42
    STATION_28_BUTTON = 43
    STATION_29_BUTTON = 44
    STATION_30_BUTTON = 45
    STATION_31_BUTTON = 46
    STATION_32_BUTTON = 47
    STATION_33_BUTTON = 48
    STATION_34_BUTTON = 49
    STATION_35_BUTTON = 50
    STATION_36_BUTTON = 51
    STATION_37_BUTTON = 52
    STATION_38_BUTTON = 53
    STATION_39_BUTTON = 54
    STATION_40_BUTTON = 55
    STATION_41_BUTTON = 56
    STATION_42_BUTTON = 57
    STATION_43_BUTTON = 58
    STATION_44_BUTTON = 59
    STATION_45_BUTTON = 60
    STATION_46_BUTTON = 61
    STATION_47_BUTTON = 62
    STATION_48_BUTTON = 63

    if (number_pages - PAGES_TEMP_OFFSET) >= 1:
        # Station IDs Page 1
        STATION_49_BUTTON = 64
        STATION_50_BUTTON = 65
        STATION_51_BUTTON = 66
        STATION_52_BUTTON = 67
        STATION_53_BUTTON = 68
        STATION_54_BUTTON = 69
        STATION_55_BUTTON = 70
        STATION_56_BUTTON = 71
        STATION_57_BUTTON = 72
        STATION_58_BUTTON = 73
        STATION_59_BUTTON = 74
        STATION_60_BUTTON = 75
        STATION_61_BUTTON = 76
        STATION_62_BUTTON = 77
        STATION_63_BUTTON = 78
        STATION_64_BUTTON = 79
        STATION_65_BUTTON = 80
        STATION_66_BUTTON = 81
        STATION_67_BUTTON = 82
        STATION_68_BUTTON = 83
        STATION_69_BUTTON = 84
        STATION_70_BUTTON = 85
        STATION_71_BUTTON = 86
        STATION_72_BUTTON = 87
        STATION_73_BUTTON = 88
        STATION_74_BUTTON = 89
        STATION_75_BUTTON = 90
        STATION_76_BUTTON = 91
        STATION_77_BUTTON = 92
        STATION_78_BUTTON = 93
        STATION_79_BUTTON = 94
        STATION_80_BUTTON = 95
        STATION_81_BUTTON = 96
        STATION_82_BUTTON = 97
        STATION_83_BUTTON = 98
        STATION_84_BUTTON = 99
        STATION_85_BUTTON = 100
        STATION_86_BUTTON = 101
        STATION_87_BUTTON = 102
        STATION_88_BUTTON = 103
        STATION_89_BUTTON = 104
        STATION_90_BUTTON = 105
        STATION_91_BUTTON = 106
        STATION_92_BUTTON = 107
        STATION_93_BUTTON = 108
        STATION_94_BUTTON = 109
        STATION_95_BUTTON = 110
        STATION_96_BUTTON = 111

    if (number_pages - PAGES_TEMP_OFFSET) >= 2:
        # Station IDs Page 2
        # Add the next station buttons STATION_97_BUTTON = 112 ... STATION_144_BUTTON = 159
        pass

    def __str__(self):
        # print(f"Current value: {self._value_}")
        return f" {self._name_}"
