rix configuration interface
# (c) 2022 Barix AG
#
# meta-file for automatic config-file generation

source /lib/config/functions.sh
SSL_CERT="/mnt/data/certs/barix.pem"

# function to regenerate the SSL certificate
function regen_certificate() {
	logger "Self-generated certificate validity check failed! Regenerating..."

  /usr/bin/openssl req -newkey rsa:2048 -x509 -days 7300 -nodes -keyout $SSL_CERT -out $SSL_CERT -subj "/emailAddress=info@barix.com/C=CH/ST=Zurich/L=Dubendorf/O=Barix AG/CN=barix.local"
  chmod 400 $SSL_CERT
	# if the device is powered off after generating the certificate, it may be lost
	# or have 0 bytes files, so do a sync just in case to flush all changes to the SD card
	sync
	sync
  cfg_set_param httpd.ssl.certificate $SSL_CERT
  /usr/bin/uci commit httpd
}

function validate_cert() {
	# It may happen sometimes during the production process the newly generated certificate
	# to be corrupted because the device is powered immediately after end of the first boot
	# process, and the changes may not be correctly flushed to the SD card, so do additional
	# check for the self generated certificate integrity, and regenerate it on the next boot
	# if it is corrupted

	/usr/bin/openssl x509 -in $SSL_CERT -noout >/dev/null 2>&1
	cert_ok=$?
	/usr/bin/openssl rsa -in $SSL_CERT -check -noout >/dev/null 2>&1
	key_ok=$?
	[ "$(/usr/bin/openssl rsa -check -noout -in $SSL_CERT)" == "RSA key ok" ] && rsa_ok="OK" || rsa_ok="NOK"

	if [ "$cert_ok" -ne 0 ] || [ "$key_ok" -ne 0 ] || [ "$rsa_ok" == "NOK" ] ; then
		regen_certificate
	fi
}

# function to create dynamic content
function create_dynamic_config_httpd()
{

	protocol=`cfg_print_param httpd.webserver.protocol`

	if [  "$protocol" == "https" ]; then
		cert=`cfg_print_param httpd.ssl.certificate`

		if [ ! -f "$cert" ]; then
			if ! [ -d /mnt/data/certs ]; then
				mkdir -p /mnt/data/certs
			fi

			# TODO:try to get it from flash

			# Auto generate SSL self-signed certificate with validity for 20 years
			# /usr/bin/openssl req -newkey rsa:2048 -x509 -days 7300 -nodes -keyout /mnt/data/certs/barix.pem -out /mnt/data/certs/barix.pem -subj "/emailAddress=info@barix.com/C=CH/ST=Zurich/L=Dubendorf/O=Barix AG/CN=barix.local"
			# chmod 400 /mnt/data/certs/barix.pem
			# cfg_set_param httpd.ssl.certificate '/mnt/data/certs/barix.pem'
			# /usr/bin/uci commit httpd
			regen_certificate
		fi

		# Always validate the https certificate
		validate_cert

	else # HTTP
		echo ''
	fi
}

create_dynamic_config_httpd
